import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  QueryList,
  ViewChild,
  ViewChildren,
  ViewEncapsulation,
} from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort, Sort } from '@angular/material/sort';
import {
  map,
  merge,
  Observable,
  of,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { User } from '../models/user';
import { UserPagination } from '../models/user-pagination';
import { UserService } from '../services/user.service';
import { ActivatedRoute, Router } from '@angular/router';
import { IdentityService } from 'app/modules/polyglot/identity/services/identity.service';
import { MatDialog } from '@angular/material/dialog';
import { CashComposeComponent } from '../compose/cash-compose.component';
import { CommissionComposeComponent } from '../compose/commission-compose.component';
import { CompComposeComponent } from '../compose/comp-compose.component';
import { CouponComposeComponent } from '../compose/coupon-compose.component';
import { MessageComposeComponent } from '../compose/message-compose.component';
import { ListMembersResponse } from 'app/modules/proto/c2se/member_pb';
import { MemberModel, MemberState } from 'app/modules/proto/models/member_pb';
import { MemberClass } from 'app/modules/proto/models/member_class_pb';
import { MemberClassService } from 'app/modules/polyglot/member_class/services/member_class.service';
import { result } from 'lodash';
import { MatCheckbox, MatCheckboxChange } from '@angular/material/checkbox';
import { ListMemberLevelsResponse } from 'app/modules/proto/c2se/member_level_pb';
import { MemberLevel } from 'app/modules/proto/models/member_level_pb';
import { ListMemberClassesResponse } from 'app/modules/proto/c2se/member_class_pb';

@Component({
  selector: 'user-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .user-grid {
        /*                     CB   관리  요율  상부  아이디 */
        grid-template-columns: 20px 40px 40px 100px auto;

        @screen sm {
          /*                     CB   관리  요율  상부  아이디 등급 */
          grid-template-columns: 20px 40px 40px 100px auto 40px 140px 140px;
        }

        @screen md {
          /*                     CB   관리  요율  상부  아이디  등급 예금주 보유금  게임중머니  총입출 */
          grid-template-columns: 20px 40px 40px 100px auto 40px 140px 140px;
        }

        @screen lg {
          /*                     CB   관리  요율  상부  아이디 등급  예금주  보유금 게임중머니  총입출 카지노 로그 */
          grid-template-columns: 20px 40px 40px 100px auto 40px 140px 140px 140px 140px 140px 140px;
        }
      }

      .mat-tooltip {
        white-space: pre-line;
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  @ViewChildren('chkUsers') chkUsers!: QueryList<MatCheckbox>;

  listMembers$!: Observable<MemberModel[] | undefined>;
  listMemberLevels$!: Observable<MemberLevel[] | undefined>;

  __isSearchOpened = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedUser?: User;
  pagination?: UserPagination;

  memberClasses!: MemberClass[];
  __checkedUsers: string[] = [];

  signoutBlockConfigForm!: FormGroup;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _activatedRoute: ActivatedRoute,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _userService: UserService,
    private _identityService: IdentityService,
    private _memberClassService: MemberClassService,
    private router: Router,
    private _matDialog: MatDialog
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._userService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: UserPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result = data['listmembers'];
      this.listMembers$ = of(listMemberResult.getMembersList());

      let listMemberLevelsResult: ListMemberLevelsResponse.Result =
        data['listMemberLevels'];
      this.listMemberLevels$ = of(listMemberLevelsResult.getMemberLevelsList());

      let listMemberClassesResult: ListMemberClassesResponse.Result =
        data['listMemberClasses'];
      this.memberClasses = listMemberClassesResult.getMemberClassesList();

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });

    // Set signout block config
    this.__signoutBlockConfirmConfig();
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the user changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._userService.getUsers(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }

  viewSlot(id: string): void {
    let url: string = 'game/slot';
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch(): void {
    this.__isSearchOpened = !this.__isSearchOpened;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickCash(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CashComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCommission(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CommissionComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickComp(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CompComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCoupon(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CouponComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickMessage(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(MessageComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __getRateTooltop(): string {
    const tempRate = 0;
    const resultTooltip = `
    요율확인 \n 카지노-바카라: ${tempRate}% \n 카지노-룰렛: ${tempRate}% \n 카지노-드레곤타이거: ${tempRate}% \n 카지노-그외:${tempRate}% \n 슬롯:${tempRate}% \n 카지노루징: ${tempRate}% \n 슬롯루징: ${tempRate}%
    `;
    return resultTooltip;
  }
  __signoutBlockConfirmConfig(): void {
    this.signoutBlockConfigForm = this._formBuilder.group({
      title: '알림',
      message: '로그아웃되었습니다.',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: false,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }
  __onClickSignoutBlock(event: MouseEvent): void {
    // Open the dialog and save the reference of it
    const dialogRef = this._fuseConfirmationService.open(
      this.signoutBlockConfigForm.value
    );
  }

  __onChangeChkUsersAll(event: MatCheckboxChange): void {
    let checked = event.checked;
    this.chkUsers.forEach((c, i) => {
      c.checked = checked;
    });
    this.__updateCheckedUsers();
  }

  __onChangeChkUsers(event: MatCheckboxChange): void {
    this.__updateCheckedUsers();
  }

  __updateCheckedUsers(): void {
    this.__checkedUsers = [];
    this.chkUsers.forEach((c, i) => {
      if (c.checked) {
        this.__checkedUsers.push(c.value);
      }
    });
  }

  __getParentUsername(member: MemberModel): string {
    if (!member || !member.getParentMember()) {
      return '';
    }

    const parentUsername = member.getParentMember()?.getUsername();
    const parentClassessId = member.getParentMember()?.getMemberClassId();

    const findClass = this.findMemberClasses(parentClassessId);

    if (!findClass) {
      return '';
    }

    return `[${findClass.getName()}]${parentUsername}`;
  }

  __onMatSortChangeUsers(sort: Sort): void {}

  __getMemberState(member: MemberModel): string {
    const state = member.getState();
    let resultStr = '';

    switch (state) {
      case MemberState.NONE:
        resultStr = '없음';
        break;
      case MemberState.NORMAL:
        resultStr = '정상';
        break;
      case MemberState.PENDING:
        resultStr = '대기';
        break;
      case MemberState.WITHDRAWAL:
        resultStr = '탈퇴';
        break;
      case MemberState.DORMANCY:
        resultStr = '휴면';
        break;
      case MemberState.BLACKLIST:
        resultStr = '블랙리스트';
        break;
      case MemberState.SUSPENDED:
        resultStr = '정지';
        break;
      default:
        break;
    }

    return resultStr;
  }

  private findMemberClasses(classId?: string): MemberClass | undefined {
    if (!classId || classId === '') {
      return undefined;
    }
    return this.memberClasses.find((mc) => mc.getId() === classId);
  }
}
