import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { BankService } from 'app/modules/polyglot/bank/services/bank.service';
import { MemberService } from 'app/modules/polyglot/member/services/member.service';
import { MemberLevelService } from 'app/modules/polyglot/member_level/services/member_level.service';
import { SiteService } from 'app/modules/polyglot/site/services/site.service';
import { ListBanksResponse } from 'app/modules/proto/c2se/bank_pb';
import { ListMemberLevelsResponse } from 'app/modules/proto/c2se/member_level_pb';
import { GetMemberResponse } from 'app/modules/proto/c2se/member_pb';
import {
  ListMembersRequest,
  ListMembersResponse,
} from 'app/modules/proto/c2se/member_pb';
import { ListSitesResponse } from 'app/modules/proto/c2se/site_pb';

import { catchError, Observable, throwError } from 'rxjs';

import { User } from '../models/user';
import { UserPagination } from '../models/user-pagination';
import { UserService } from '../services/user.service';

@Injectable({
  providedIn: 'root',
})
export class UserResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _userService: UserService, private _router: Router) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<User | undefined> {
    return this._userService.getUserById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class UsersResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _userService: UserService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: UserPagination;
    users: User[];
  }> {
    return this._userService.getUsers();
  }
}

@Injectable({
  providedIn: 'root',
})
export class ListMemberResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _router: Router, private _memberService: MemberService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */

  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<ListMembersResponse.Result>
    | Promise<ListMembersResponse.Result>
    | ListMembersResponse.Result {
    return this._memberService.listMembers();
  }
}

@Injectable({
  providedIn: 'root',
})
export class MemberResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _router: Router, private _memberService: MemberService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */

  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<GetMemberResponse.Result>
    | Promise<GetMemberResponse.Result>
    | GetMemberResponse.Result {
    let memberId = route.paramMap.get('id');
    if (!memberId) {
      console.error('user.resolver memberId is null');
    }
    return this._memberService.getMember(memberId!);
  }
}

export class SitesResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private __siteService: SiteService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<ListSitesResponse.Result>
    | Promise<ListSitesResponse.Result>
    | ListSitesResponse.Result {
    return this.__siteService.listSites();
  }
}

export class BanksResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _bankService: BankService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<ListBanksResponse.Result>
    | Promise<ListBanksResponse.Result>
    | ListBanksResponse.Result {
    return this._bankService.listBanks();
  }
}
